/*= R&S OSP Open Switch and Control Platform ================================*/
/*===========================================================================*/
/*  Please do not use global variables or arrays in the include file of      */
/*  instrument drivers that will be submitted for inclusion into the         */
/*  LabWindows Instrument Driver Library.                                    */
/*===========================================================================*/

#ifndef __RSOSP_HEADER
#define __RSOSP_HEADER

#include <visa.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************/
/*= Instrument Driver Specific Error/Warning Codes ==========================*/
/*****************************************************************************/
#define VI_WARN_NSUP_ID_QUERY     (          0x3FFC0101L)
#define VI_WARN_NSUP_RESET        (          0x3FFC0102L)
#define VI_WARN_NSUP_SELF_TEST    (          0x3FFC0103L)
#define VI_WARN_NSUP_ERROR_QUERY  (          0x3FFC0104L)
#define VI_WARN_NSUP_REV_QUERY    (          0x3FFC0105L)

#define VI_ERROR_FAIL_ID_QUERY    (_VI_ERROR+0x3FFC0011L)
#define VI_ERROR_INV_RESPONSE     (_VI_ERROR+0x3FFC0012L)

#define VI_ERROR_INSTR_FILE_OPEN                (_VI_ERROR+0x3FFC0800L)
#define VI_ERROR_INSTR_FILE_WRITE               (_VI_ERROR+0x3FFC0801L)
#define VI_ERROR_INSTR_FILE_READ                (_VI_ERROR+0x3FFC0802L)
#define VI_ERROR_INSTR_INTERPRETING_RESPONSE    (_VI_ERROR+0x3FFC0803L)
#define VI_ERROR_INSTR_PARAMETER                (_VI_ERROR+0x3FFC0810L)

/*****************************************************************************/
/*= Define Instrument Specific Error/Warning Codes Here =====================*/
/*****************************************************************************/
#define VI_WARNING_INSTR_OFFSET                          (0x3FFC0900L)
#define VI_ERROR_INSTR_OFFSET                  (_VI_ERROR+0x3FFC0900L)

#define RSSMF_ERROR_INSTRUMENT_ERROR           (VI_ERROR_INSTR_OFFSET + 0xF0L)
/* Add instrument driver specific error codes here */

#define RSSMF_ERROR_INVALID_CONFIGURATION      (VI_ERROR_INSTR_OFFSET + 0xF1L)
#define RSSMF_ERROR_INSTRUMENT_OPTION          (VI_ERROR_INSTR_OFFSET + 0xF2L)
#define RSSMF_ERROR_INSTRUMENT_NSUP_MODEL      (VI_ERROR_INSTR_OFFSET + 0xF3L)
#define RSSMF_ERROR_SETTINGS_CONFLICT          (VI_ERROR_INSTR_OFFSET + 0xF4L)
#define RSSMF_ERROR_CALIBRATION_FAILED         (VI_ERROR_INSTR_OFFSET + 0xF5L)
/* Add instrument driver specific warning codes here */
#define RSSMF_WARNING_SMALL_BUFFER             (VI_WARNING_INSTR_OFFSET + 0xF1L)
#define RSSMF_WARNING_NOT_SUPPORTED            (VI_WARNING_INSTR_OFFSET + 0xF2L)
/*****************************************************************************/
/*= Instrument Driver Specific Error/Warning Codes ==========================*/
/*****************************************************************************/
#define VI_ERROR_INSTR_FILE_OPEN                    (_VI_ERROR+0x3FFC0800L)
#define VI_ERROR_INSTR_FILE_WRITE                   (_VI_ERROR+0x3FFC0801L)
#define VI_ERROR_INSTR_INTERPRETING_RESPONSE        (_VI_ERROR+0x3FFC0803L)
    
/*****************************************************************************/
/*= Define Instrument Specific Error/Warning Codes Here =====================*/
/*****************************************************************************/
#define VI_WARNING_INSTR_OFFSET                     (0x3FFC0900L)
#define VI_ERROR_INSTR_OFFSET                       (_VI_ERROR+0x3FFC0900L)

#define RSOSP_ERROR_INSTRUMENT_ERROR                (VI_ERROR_INSTR_OFFSET + 0xF0L)
/* Add instrument driver specific error codes here */
#define RSOSP_ERROR_INVALID_CONFIGURATION           (VI_ERROR_INSTR_OFFSET + 0xF1L)

/* Add instrument driver specific warning codes here */
#define RSOSP_WARNING_SMALL_BUFFER                  (VI_WARNING_INSTR_OFFSET + 0xF1L)


/*****************************************************************************/
/*= Instrument specific defines =============================================*/
/*****************************************************************************/

/* --- Instrument Name ----------------------------------------------------- */
#define RSOSP_INSTRUMENT_F01                                1
#define RSOSP_INSTRUMENT_F02                                2
#define RSOSP_INSTRUMENT_F03                                3
#define RSOSP_INSTRUMENT_F04                                4
#define RSOSP_INSTRUMENT_F05                                5
#define RSOSP_INSTRUMENT_F06                                6
#define RSOSP_INSTRUMENT_F07                                7
#define RSOSP_INSTRUMENT_F08                                8
#define RSOSP_INSTRUMENT_F09                                9
    
/* --- Port Name ----------------------------------------------------------- */
#define RSOSP_PORT_A11                                      1
#define RSOSP_PORT_A12                                      2
#define RSOSP_PORT_A13                                      3

/* --- Registers ----------------------------------------------------------- */
#define RSOSP_REGISTER_OPERATION                            0
#define RSOSP_REGISTER_QUESTIONABLE                         1
#define RSOSP_REGISTER_ENABLE                               2
#define RSOSP_REGISTER_NTR                                  3
#define RSOSP_REGISTER_PTR                                  4

/* --- Register Type ------------------------------------------------------- */
#define RSOSP_REGISTER_COND                                 0
#define RSOSP_REGISTER_EVENT                                1
    
    
/*****************************************************************************/
/*= GLOBAL USER-CALLABLE FUNCTION DECLARATIONS (Exportable Functions) =======*/
/*****************************************************************************/
    
ViStatus _VI_FUNC rsosp_init (ViRsrc resourceName, ViBoolean IDQuery,
                              ViBoolean resetDevice, ViSession *instrumentHandle);

ViStatus _VI_FUNC rsosp_SetListOfRelayConditions (ViSession instrumentHandle,
                                                  ViString channelList);

ViStatus _VI_FUNC rsosp_GetListOfRelayConditions (ViSession instrumentHandle,
                                                  ViString channelList,
                                                  ViInt32 arraySize,
                                                  ViUInt32 _VI_FAR conditions[]);

ViStatus _VI_FUNC rsosp_SetRelayCondition (ViSession instrumentHandle,
                                           ViInt32 instrumentName,
                                           ViInt32 moduleName,
                                           ViUInt32 condition,
                                           ViUInt32 relayNumber);

ViStatus _VI_FUNC rsosp_GetRelayCondition (ViSession instrumentHandle,
                                           ViInt32 instrumentName,
                                           ViInt32 moduleName,
                                           ViUInt32 condition,
                                           ViUInt32 relayNumber,
                                           ViUInt32 *queryCondition);

ViStatus _VI_FUNC rsosp_SetRelayDelay (ViSession instrumentHandle,
                                       ViInt32 delayTime, ViString channelList);

ViStatus _VI_FUNC rsosp_GetRelayDelay (ViSession instrumentHandle,
                                       ViString channelList, ViInt32 arraySize,
                                       ViInt32 _VI_FAR delayTime[]);

ViStatus _VI_FUNC rsosp_SetOSPName (ViSession instrumentHandle,
                                       ViString instrumentName,
                                       ViString instrumentAddress);

ViStatus _VI_FUNC rsosp_GetOSPAddress (ViSession instrumentHandle,
                                       ViString instrumentName,
                                       ViInt32 bufferSize,
                                       ViChar _VI_FAR instrumentAddress[]);

ViStatus _VI_FUNC rsosp_DeleteOSP (ViSession instrumentHandle,
                                      ViString instrumentName);

ViStatus _VI_FUNC rsosp_DeleteAllOSPs (ViSession instrumentHandle);

ViStatus _VI_FUNC rsosp_UpdateInternalData (ViSession instrumentHandle);

ViStatus _VI_FUNC rsosp_OSPCatalog (ViSession instrumentHandle,
                                       ViInt32 arraySize, 
                                       ViChar _VI_FAR catalogList[]);

ViStatus _VI_FUNC rsosp_SetPathDefinition (ViSession instrumentHandle, ViString pathName,
                                 			ViString channelList);

ViStatus _VI_FUNC rsosp_GetPathDefinition (ViSession instrumentHandle, ViString pathName,
                                 			ViInt32 arraySize, 
                                 			ViChar _VI_FAR channelList[]);
ViStatus _VI_FUNC rsosp_SwitchPath (ViSession instrumentHandle,
                                     ViString pathName);
ViStatus _VI_FUNC rsosp_DeletePathDefinition (ViSession instrumentHandle,
                                    			ViString pathName);

ViStatus _VI_FUNC rsosp_DeleteAllPathDefinitions (ViSession instrumentHandle);

ViStatus _VI_FUNC rsosp_PathCatalog (ViSession instrumentHandle,
                                     ViInt32 arraySize, 
                                     ViChar _VI_FAR catalogList[]);

ViStatus _VI_FUNC rsosp_ReadOperationCycles (ViSession instrumentHandle,
                                             ViString channelList,
                                             ViInt32 arraySize,
                                             ViInt32 _VI_FAR operationCycles[]);

ViStatus _VI_FUNC rsosp_ReadInterlockState (ViSession instrumentHandle,
                                            ViString moduleName,
                                            ViBoolean *interlockState);

ViStatus _VI_FUNC rsosp_ReadInputChannelsCondition (ViSession instrumentHandle,
                                 		              ViString moduleName,
                                        		       ViUInt32 *channelsCondition);

ViStatus _VI_FUNC rsosp_InternalSelfTest (ViSession instrumentHandle,
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR selfTestResults[]);

ViStatus _VI_FUNC rsosp_HardwareInformation (ViSession instrumentHandle,
                                             ViInt32 bufferSize,
                                             ViChar _VI_FAR hwInformation[]);

ViStatus _VI_FUNC rsosp_MMemoryCatalog (ViSession instrumentHandle,
                                        ViInt32 arraySize, 
                                        ViChar _VI_FAR catalogList[]);

ViStatus _VI_FUNC rsosp_MMemoryDelete (ViSession instrumentHandle,
                                       ViString fileName);

ViStatus _VI_FUNC rsosp_MMemoryLoad (ViSession instrumentHandle,
                                     ViInt32 instrumentSetting,
                                     ViString fileName);

ViStatus _VI_FUNC rsosp_MMemoryStore (ViSession instrumentHandle,
                                      ViInt32 instrumentSetting,
                                      ViString fileName);

ViStatus _VI_FUNC rsosp_SetSystemKeyboardLock (ViSession instrumentHandle,
                                               ViBoolean keyboardLock);

ViStatus _VI_FUNC rsosp_GetSystemKeyboardLock (ViSession instrumentHandle,
                                               ViBoolean *keyboardLock);

ViStatus _VI_FUNC rsosp_SystemPreset (ViSession instrumentHandle);

ViStatus _VI_FUNC rsosp_GetSystemVersion (ViSession instrumentHandle,
                                          ViChar SCPIVersion[]);

ViStatus _VI_FUNC rsosp_errorCheckState (ViSession instrumentHandle,
                                         ViBoolean stateChecking);

ViStatus _VI_FUNC rsosp_optionCheckState (ViSession instrumentHandle,
                                          ViBoolean stateChecking);

ViStatus _VI_FUNC rsosp_writeInstrData (ViSession instrumentHandle,
                                        ViString writeBuffer);

ViStatus _VI_FUNC rsosp_readInstrData (ViSession instrumentHandle,
                                       ViInt32 numberBytesToRead,
                                       ViChar _VI_FAR readBuffer[],
                                       ViInt32 *numBytesRead);

ViStatus _VI_FUNC rsosp_reset (ViSession instrumentHandle);

ViStatus _VI_FUNC rsosp_self_test (ViSession instrumentHandle,
                                   ViInt16 *selfTestResult,
                                   ViChar _VI_FAR selfTestMessage[]);

ViStatus _VI_FUNC rsosp_error_query (ViSession instrumentHandle,
                                     ViInt32 *errorCode, 
                                     ViChar _VI_FAR errorMessage[]);

ViStatus _VI_FUNC rsosp_error_message (ViSession instrumentHandle,
                                       ViStatus statusCode, 
                                       ViChar _VI_FAR message[]);

ViStatus _VI_FUNC rsosp_revision_query (ViSession instrumentHandle,
                                        ViChar _VI_FAR driverRev[],
                                        ViChar _VI_FAR instrRev[]);

ViStatus _VI_FUNC rsosp_close (ViSession instrumentHandle);

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif

/*****************************************************************************/
/*=== END INCLUDE FILE ======================================================*/
/*****************************************************************************/

#endif
