#include <visa.h>

/*= Useful Macros ===========================================================*/

/* - Basic Macros ---------------------------------------------------------- */

/* Error checking macro */
#define CHECKERR(fCal) if (rsosp_status = (fCal), rsosp_status < VI_SUCCESS) \
    return rsosp_status

/* Error checking macro with jump to the ERROR: label */
#define GOCHECKERR(fCal) if (rsosp_status = (fCal), rsosp_status < VI_SUCCESS) \
                            goto ERROR_LABEL

/* Parameter checking macro */
#define CHECKPARAM(fCal,param) if (fCal != VI_SUCCESS) return VI_ERROR_PARAMETER##param

/* Initialize default function parameters */
#define INIT\
    ViStatus            rsosp_status = VI_SUCCESS; \
    rsosp_instrRange    instrPtr = NULL; \
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));\
    if (instrPtr == NULL)\
        return VI_ERROR_PARAMETER1

/* Close function and return */
#define RETURN\
    return rsosp_status;

/* - Additional Macros ----------------------------------------------------- */
/* Remap parameter error code */
#define REMAP(par_in, par_out) if (rsosp_status == VI_ERROR_PARAMETER##par_in)\
                                    rsosp_status = VI_ERROR_PARAMETER##par_out;\
                                if (rsosp_status < VI_SUCCESS)\
                                    return rsosp_status

#define REMAPINSTR(par_in, par_out) if (rsosp_status == VI_ERROR_PARAMETER##par_in)\
                                    rsosp_status = VI_ERROR_INSTR_PARAMETER##par_out;\
                                if (rsosp_status < VI_SUCCESS)\
                                    return rsosp_status

/* Check NULL pointer */
#define CHECKNULL(pointer) if (pointer == NULL) return rsosp_ParameterError(instrSession,#pointer)

#define	RSOSP_INSTRUMENT_COUNT			10

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* rsosp_stringValPair is used in the rsosp_error_message function           */
/* rsosp_statusDataRanges is used to track session dependent status & ranges */
/*===========================================================================*/
typedef struct  rsosp_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  rsosp_tStringValPair;

enum
{
    UNKNOWN = -1,
    B101 = 101,
    B102 = 102,
    B103 = 103,
	B104 = 104,
	B105 = 105,	
	B115 = 115,	
};

typedef struct rsosp_instrPorts
{
    int A11;
    int A12;
    int A13;
} rsosp_tinstrPorts;

struct rsosp_statusDataRanges
{
    ViChar              instrDriverRevision[256];
    ViBoolean           errorChecking;
	ViBoolean       	optionChecking;
    rsosp_tinstrPorts 	instrPorts[RSOSP_INSTRUMENT_COUNT];
	ViBoolean           rangeChecking;
	ViChar      parameterErrorMessage[256];
};

typedef struct rsosp_statusDataRanges *rsosp_instrRange;
ViBoolean rsosp_invalidViInt32Range(ViSession instrSession, ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean rsosp_invalidViUInt32Range(ViSession instrSession, ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean rsosp_invalidViReal64Range(ViSession instrSession, ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus rsosp_checkStatus (ViSession instrSession);
ViStatus rsosp_checkOption (ViSession instrSession, ViString channelList, ViInt32 option);
ViStatus rsosp_sendCmd (ViSession instrSession, ViString cmd);
ViStatus rsosp_setViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 value,
                           ViInt32 min,
                           ViInt32 max,
                           ViString parameterName);
ViStatus rsosp_getViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 *value,
                           ViString parameterName);
ViStatus rsosp_getViUInt32 (ViSession instrSession,
                            ViString cmd,
                            ViUInt32 *value,
                            ViString parameterName);
ViStatus rsosp_setViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean value);
ViStatus rsosp_getViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean *value,
                           ViString parameterName);
ViStatus rsosp_setViReal64 (ViSession vi,
                            ViString cmd,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViString parameterName);
ViStatus rsosp_getViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViString parameterName);
ViStatus rsosp_setViReal64Units (ViSession instrSession,
                                 ViString cmd,
                                 ViReal64 value,
                                 ViString units,
                                 ViReal64 min,
                                 ViReal64 max,
                                 ViString parameterName);
ViStatus rsosp_setViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
							ViString parameterName);
ViStatus rsosp_getViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViUInt32 buffer_size,
                            ViString parameterName);
ViStatus rsosp_setViStringQuoted (ViSession instrSession,
                            ViString cmd,
                            ViString quotes,
                            ViString value,
							ViString parameterName);
ViStatus rsosp_setEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 value,
                        ViString list[],
                        ViString parameterName);
ViStatus rsosp_getEnum (ViSession instrSession,
                        ViString cmd,
                        ViInt32 *value,
                        ViString list[],
                        ViString parameterName);
ViStatus rsosp_checkIndexInList (ViSession instrSession,
								 ViInt32 index,
                                 ViString list[],
								 ViString parameterName);
ViStatus rsosp_getIndexFromString (ViString string,
                                   ViString *list,
                                   ViInt32 *index);
ViStatus rsosp_getCatalog (ViSession instrSession,
                           ViString cmd,
                           ViUInt32 arraySize,
                           ViChar* array,
                           ViString parameterName);
ViStatus rsosp_getViReal64Array (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViInt32  count,
                            ViString parameterName);
ViStatus rsosp_getViInt32Array (ViSession instrSession,
                            ViString cmd,
                            ViInt32 *value,
                            ViInt32  count,
                            ViString parameterName);
ViStatus rsosp_getViUInt32Array (ViSession instrSession,
                             ViString cmd,
                             ViUInt32 *value,
                             ViInt32  count,
                             ViString parameterName);
ViStatus rsosp_readDataUnknownLength(ViSession instrSession,
                                     /*@null@*/ViChar **outBuffer,
                                     ViUInt32 *count);
void RemoveSurroundingWhiteSpace(ViChar string[]);
ViStatus rsosp_ParameterError(ViSession io, ViString parameterName);
